import os
import jwt

from jwt.exceptions import PyJWTError
from werkzeug.exceptions import Unauthorized
from ingaia.gcloud import StorageUtil

from ingaia.gcloud import ProjectReference

RSA_PROJECT_ID = 'ingaia-authentication-api'
RSA_BUCKET_LOCATION = ''  # TODO
RSA_BUCKET = 'authentication-rsa'
RSA_PUBLIC_KEY = 'jwt-token-key.pub'
RSA_PROJECT_REF = ProjectReference(RSA_PROJECT_ID, RSA_BUCKET_LOCATION)

public_key = StorageUtil(RSA_BUCKET, RSA_PUBLIC_KEY, project=RSA_PROJECT_REF).get_content()


def decode_token(token):
    """
    Function responsible for decode the token informed by the client

    :param token:   JWT Token
    :return:        json decoded
    :raise:         Unauthorized when token is invalid
    """
    try:
        return jwt.decode(token, public_key, algorithms=['RS256'])
    except PyJWTError as e:
        raise Unauthorized()



"""------------------------------
inGaia python utility library
------------------------------
Version: {__version__}
"""
import logging
from ingaia import __version__


def main(*args, **kwargs):
    print(__doc__.replace('{__version__}', __version__))


if __name__ == "__main__":
    main()

    try:
        from ingaia.request_logger.request_logger import generate_rl_token
        from ingaia.request_logger.decode import decode_token

        rl_token = generate_rl_token(f'ingaia-libs{__version__}')
        print(rl_token)
        print(decode_token(rl_token))
    except Exception as e:
        print(f'Request log token generation error: {e}')



import os
import requests
import base64


class AuthUtil:
    TOKEN_URL = 'https://ingaia-authentication-api.appspot.com/operations/auth/token'

    APP_ID = os.environ.get('AUTH_APP_ID')
    APP_SECRET = os.environ.get('AUTH_APP_SECRET')

    @staticmethod
    def get_token():
        # concat app id with secret
        app_id_plus_secret = f'{AuthUtil.APP_ID}:{AuthUtil.APP_SECRET}'
        b64_user_pass = base64.b64encode(app_id_plus_secret.encode('latin1'))
        b64_user_pass_str = b64_user_pass.decode('latin1')

        headers = {
            'Authorization': f'Basic {b64_user_pass_str}',
            'Content-Type': 'application/x-www-form-urlencoded',
        }

        data = {'grant_type': 'client_credentials'}

        return requests.post(AuthUtil.TOKEN_URL, data=data, headers=headers).json()

    @staticmethod
    def get_authenticated_header():
        return {'Authorization': 'Bearer {}'.format(AuthUtil.get_token())}


if __name__ == '__main__':
    print(AuthUtil.get_token())



from ingaia.gcloud import ProjectReference, StorageUtil
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.serialization import load_pem_private_key


class RequestLoggerConfig:
    # constants
    DB_STORE = True
    REQUEST_LOGGER_DATA_STORE_URL = 'https://us-central1-ingaia-request-logger.cloudfunctions.net/save_request_log'

    # Google cloud constants
    GOOGLE_CLOUD_PROJECT_ID = 'ingaia-request-logger'
    GOOGLE_CLOUD_DEFAULT_LOCATION = 'southamerica-east1'
    GOOGLE_CLOUD_PROJECT = ProjectReference(GOOGLE_CLOUD_PROJECT_ID, GOOGLE_CLOUD_DEFAULT_LOCATION)

    # Authentication
    OPERATIONS_APP_CLIENT = 'ingaia_operations'  # Id from Google Firestore
    JWT_ISSUER = 'br.com.inagaia.operations'  # Name of the issuer
    JWT_LIFETIME_SECONDS = 3600  # 1 hour
    JWT_ALGORITHM = 'RS256'

    # queue configuration
    REQUEST_LOG_TOPIC_NAME = 'trace'

    RSA_BUCKET_LOCATION = 'us'
    RSA_BUCKET = 'ingaia-request-logger-rsa'
    RSA_PRIVATE_KEY = 'rl-token-key'
    RSA_PUBLIC_KEY = 'rl-token-key.pub'
    RSA_PROJECT_REF = ProjectReference(GOOGLE_CLOUD_PROJECT_ID, RSA_BUCKET_LOCATION)

    @property
    def JWT_SECRET(self):
        # configure the private key
        private_key = StorageUtil(self.RSA_BUCKET, self.RSA_PRIVATE_KEY, project=self.RSA_PROJECT_REF).get_content()
        # private_key = str(file_key).encode('utf-8')
        return load_pem_private_key(private_key, password=b'inGaia,2019', backend=default_backend())

    @classmethod
    def enable_database_store(cls):
        cls.DB_STORE = True

    @classmethod
    def disable_database_store(cls):
        cls.DB_STORE = False


config = RequestLoggerConfig()



import jwt

from jwt.exceptions import PyJWTError
from werkzeug.exceptions import Unauthorized
from .config import config
from ingaia.gcloud import StorageUtil


def decode_token(token):
    """
    Function responsible for decode the token informed by the client

    :param token:   JWT Token
    :return:        json decoded
    :raise:         Unauthorized when token is invalid
    """
    try:
        public_key = StorageUtil(config.RSA_BUCKET, config.RSA_PUBLIC_KEY, project=config.RSA_PROJECT_REF).get_content()
        return jwt.decode(token, public_key, algorithms=['RS256'])
    except PyJWTError as e:
        raise Unauthorized()
